require 'digest/md5'

Puppet::Parser::Functions.newfunction(:postgresql_password, :type => :rvalue, :doc => <<-'ENDHEREDOC') do |args|
  Generates a crypted password when given a username and a clear text
  password. The crypted version is generated by concatenating the clear text
  password with the username and generating the md5 checksum of the new
  string. The resulting value can be used to easily enforce a user's
  password with a `pg_role` resource as this resource expects the provided
  password value to be already encrypted.
ENDHEREDOC

  if args.size != 2
    raise ArgumentError, "Must pass two values: The first has to be the username and the second the password in plain text"
  end
  'md5' + Digest::MD5.hexdigest(args[1].to_s + args[0].to_s)
end
